import sys

from pydantic import ValidationError
from tabulate import tabulate

from ipc.utility.helper import schema_correction
from ipc.validator.schemas.dataset_metadata_schemas import build_schema_for_repo


def check_metadata_for_errors(repo, desc_df, metadata, level='value', print_table=False):
    """
    Utility to return errors for a single sample level metadata dict
    Args:
        desc_df:
        metadata: dict containing the sample level metadata to be checked
        level: level of checks to do, level='schema', 'value'
        print_table: flag to print the results as a table or not

    Returns:
        errors and desc :  errors as an error df
        status (boolean): Status for if the given metadata passed all checks or not
    """
    error_df = pd.DataFrame(columns=['Field', 'Error Message', 'Error Type', 'Repo',
                                     'dataset_id', 'key'])
    additional_errors = pd.DataFrame(columns=['Field', 'Error Message', 'Error Type', 'Repo',
                                              'dataset_id', 'key'])
    collected_errors = pd.DataFrame(columns=['Field', 'Error Message', 'Error Type', 'Repo',
                                             'dataset_id', 'key'])
    try:
        schema, schema_with_validators = build_schema_for_repo(desc_df)
        if not schema or not schema_with_validators:
            raise Exception(f'Error building pydantic classes. {tb.format_exc()}')
        if level == 'schema':
            try:
                schema(**metadata)
            except ValidationError as schema_errors:
                error_df = pd.DataFrame(schema_errors.errors())
                # Rename columns to our desired names
            if not error_df.empty:
                error_df.rename(columns={'loc': 'Field',
                                         'msg': 'Error Message',
                                         'type': 'Error Type'}, inplace=True)
                error_df['Repo'] = repo
                if 'dataset_id' in metadata:
                    if isinstance(metadata['dataset_id'], list):
                        error_df['dataset_id'] = metadata['dataset_id'][0]
                    else:
                        error_df['dataset_id'] = metadata['dataset_id']
                else:
                    error_df['dataset_id'] = 'NA'
                error_df['key'] = metadata['key']
                error_df['Field'] = error_df['Field'].apply(lambda row: row[0])
                error_df.drop_duplicates(inplace=True)
                if print_table:
                    # 'print_table' is True: returns formatted table for the command line
                    print(tabulate(error_df, headers="keys", tablefmt="fancy_grid"))
                return error_df, False
        elif level == 'value':
            """Step 1. Check only for schema"""
            try:
                schema(**metadata)
            except ValidationError as schema_errors:
                error_df = pd.DataFrame(schema_errors.errors())
                # Rename columns to our desired names
                error_df.rename(columns={'loc': 'Field',
                                         'msg': 'Error Message',
                                         'type': 'Error Type'}, inplace=True)
                errors_to_correct = schema_errors.errors()
                # Perform intermediate in-memory corrections
                if errors_to_correct:
                    metadata = schema_correction(metadata, errors_to_correct)

            """Step 2. Check for value checks with corrected schema"""
            try:
                schema_with_validators(**metadata)
            except ValidationError as value_errors:
                additional_errors = pd.DataFrame(value_errors.errors())
                # Rename columns to our desired names
                additional_errors.rename(columns={'loc': 'Field',
                                                  'msg': 'Error Message',
                                                  'type': 'Error Type'}, inplace=True)
            # If some errors are found in any of the two checks. Concatenate them together.
            if not error_df.empty and not additional_errors.empty:
                collected_errors = pd.concat([error_df, additional_errors], ignore_index=True)
                collected_errors.drop_duplicates(inplace=True)
            elif not error_df.empty and additional_errors.empty:
                collected_errors = error_df.copy()
            elif error_df.empty and not additional_errors.empty:
                collected_errors = additional_errors.copy()
            collected_errors['Repo'] = repo
            if 'dataset_id' in metadata:
                if isinstance(metadata['dataset_id'], list):
                    collected_errors['dataset_id'] = metadata['dataset_id'][0]
                else:
                    collected_errors['dataset_id'] = metadata['dataset_id']
            else:
                collected_errors['dataset_id'] = 'NA'
            collected_errors['key'] = metadata['key']
            collected_errors['Field'] = collected_errors['Field'].apply(lambda row: row[0])
            if print_table:
                # 'print_table' is True: returns formatted table for the command line
                print(tabulate(collected_errors, headers="keys", tablefmt="fancy_grid"))
            return collected_errors, False
        else:
            raise Exception(f'Invalid value for argument "level": {level} ')
    except Exception as e:
        print(tb.format_exc())
        exception_type, exception_object, exception_traceback = sys.exc_info()
        filename = exception_traceback.tb_frame.f_code.co_filename
        line_number = exception_traceback.tb_lineno
        print("Exception type: ", exception_type)
        print("File name: ", filename)
        print("Line number: ", line_number)
        return pd.DataFrame(), False
    return collected_errors, True


if __name__ == '__main__':
    from polly.auth import Polly
    from polly.omixatlas import OmixAtlas
    import traceback as tb
    import pandas as pd

    pd.reset_option('display.max_columns')
    # Obtain authentication tokens
    AUTH_TOKEN = 'eyJjdHkiOiJKV1QiLCJlbmMiOiJBMjU2R0NNIiwiYWxnIjoiUlNBLU9BRVAifQ.eAxEF4Qy8PZPMx4hVBPSl4Amd-OOQA0Iy57YeyQAGthLcy1_vW5THhUJ1bhlNRDwBeg_8svM30evW261113ugFFodYkYZAmNyVnzH7drs35JZ4iM0Daa9_afxMxJhgJtz7Y0uWCakMhFkZtKBsvX96fa57MnrlkJ_h4yzLTNC7sW_Wk4a2O9KMKbgC_EcwhBqZSaoCRt3CsTfudSoFgBe3J5e1_Ouq6PUbx7cGta2ozrc3kW8-bj6XbMnHApG09Sn090ilflNs8mtGONfz22turuoC_Q9AypAs_ZRgqBkoPHFnHn-BdZ3Y6qJK0AtIqlAHBpVs8GFDOpkeEMFH832w.jnil8O3I6DgExVTs.JLhRMl2_g4zKt5XvSs0r2xoTJDXndJ4z58IyR7CTOwe2H_OktgMDXd1SNvz1scLfP6t8BNfwNv5xnUjbRAdBk3y0bSXlcttA5vxnw3dA8yQhxgDjMy7ZNhqxDO9XwuNOr8XYQV1ob7clipciWplI4RJx3BdJOcWEuaKyQRN9VvG4qdQ6tHDeEInjsc5CX9tWK7oLsIb0rrELU2x3NEXb0Ch3mQHSbbUG9Azvu7I6zpB7uI7AW-nnfR1DWf2V4_gRo3SHn2pgGsEvOkx-AvDKPyjGAcGClthO0Zp-DOI-Yf_j0ukDzSS3GfQvJWOiW-0EsJeGHC-SnEdM4lQIfW8iQZd2upshSu_zo6Jap6Fl2KLZOjPsTH7KuPx99JMPeZBH_ZJSP7-3g6H8nX3H2qhMuZBq3qtGiz2TZUW2TaVOG3QCLBg3yKXPmbfqVdKIFLK110a3gUWNyQCx89o4AKzC3V-D1n0ZlnesiN3qOgeR602dmwLjsVNTCPy0x8P4oJDMc7vZ9Usna2k33yzT8948SbFRShbY_ZnamYM33zfKqdWS6BF6Fmi6gcRm3IA9abAa8YdbwjOONmDUORd2g1ZyzuzWTZdPYni-D4_dVgPNLGnWWlQsqFuvL_G3UitrQ_5Qs0Hm0x1SpVI5rSP8_PsM_GlOojlQW2hFss45yvvCglT7D5pA94zhnm2r2AtbNQ9n9PxpLpmZJb2G0oQZLsPTD9IPLAASaN-V_Q0DOL4pHKkPd1Bzk_IZsoIktW7ryGOqZ5twHDbspLgaa6OMEX2PS1L4sT64-_NGZl0o6igmMnHODzZSEBKXzHk1QESfHtjsyrRoX2iNYW2KsuYXcRpWhdD0W4NFLXYz7ZBomIChFY8lQOrGlbkgg_6D81Tc76mwo7Dat_cgCNTV_kQUsA7PuNgw8ukC6qEJ6g7rrvlrHoACYfTGZvgOIKpr6JuwHL2LyB8dZLnI2c6EftZJ6NymXSDJi8dUV7ydY4LzGmNM-cSi1jz28GvVzuG-7Qs-y4MfC6oCuFeWk76_3y3Rl3h7FhYMd03OXLP_sb3JXU1AuSzRGvDBTNlDytG419XXUHwgjK4E-FC-5uhAwUWTYcjLjr-Qg2qctGGctAiEvOnmdoKoGITm1OkbPWk5CnKZyzQq5uwSuJwciYRL1w7QUiu5stXKQ1Gzsf6alr2srHvr1pgbqlKupgm9iHoBqvkdAobj2lr78W6lBKpR2mHRm6sSlHFvBe-N_62b4Jvr5hkjiqzcQtYIqTrRPqc5dQdXp0VAT8jEX57422y9mwS-p758lsJd4zKrkRxYffNKZ-Q7sH6YXCnKQVnpitQg29E.iBod3im8hR4D2BdMxaCWBw'
    # Polly.auth(os.environ['AUTH_TOKEN'])
    Polly.auth(AUTH_TOKEN)

    # Defining OMIXATLAS object
    omixatlas = OmixAtlas()

    query = "SELECT * FROM depmap.datasets LIMIT 10"
    results = omixatlas.query_metadata(query, query_api_version="v2")
    dataset_metadata_test_sample = results.iloc[0].to_dict()

    query = "DESCRIBE depmap.datasets"
    schema_df = omixatlas.query_metadata(query, query_api_version="v2")

    errors, status = check_metadata_for_errors('depmap', schema_df, dataset_metadata_test_sample, print_table=True)
